/************************************************************************
 *   IRC - Internet Relay Chat, src/m_help.c
 *   Copyright (C) 2001 Nicolas Brard Nault
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* This IRCd needed a good help system. Here it is ! */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "dich_conf.h"
#include "fdlist.h"
#include "h.h"
#include <string.h>

/* You recognize DF467 there ;) */
#define HDR(str) sendto_one(sptr, rpl_str(RPL_HELPHDR), me.name, sptr->name, str)
#define SND(str) sendto_one(sptr, rpl_str(RPL_HELPSND), me.name, sptr->name, str)
#define FTR(str) sendto_one(sptr, rpl_str(RPL_HELPFTR), me.name, sptr->name, str)
#define HLP(str) sendto_one(sptr, rpl_str(RPL_HELPHLP), me.name, sptr->name, str)

void send_offhlp(aClient *sptr) {
 aChannel *chptr;
 int hashnum, i = 0;

 HLP("-- GXPIRCd help system --");
 SND("Official Help Channels List");
 for (hashnum = 1; hashnum < CH_MAX; hashnum++) {
  for (chptr = (aChannel *)hash_get_chan_bucket(hashnum);
   chptr; chptr = chptr->hnextch) {

   if (chptr && (chptr->mode.mode & MODE_OFFHLP)) {
    i++;
    sendto_one(sptr, ":%s 291 %s :%d- %s %s", me.name, sptr->name,
     i, chptr->chname, chptr->topic);
   }
  }
 }
 if (i == 0) {
  SND("There is no official help channel on the network at the moment");
 }
 HLP("----");
}

int send_help(aClient *sptr, int what) {
 int i;

 if (what == 1) {
  HDR("-- GXPIRCd help system --");
  SND("Usage: HELP [command] <subcommand> ... ");
  SND("-");
  SND("Here is a list of available commands:");
  SND("UMODES        Help with user modes");
  SND("CMODES        Help with channel modes");
  SND("WHO           Help with the who system");
  SND("DCCALLOW      Help with the dccallow command");
  SND("STATS         Help with the statistical system");
  SND("CMDS          List of commands (user/server)");
  SND("LIST          Help with the list command");
  SND("HELPCHANS     List official help channels");
  SND("VFLAGS        Learn more about flags in /version");
  SND("NEWSTUFF      New modes and functions");
  SND("WHATSNEW      Changes in this ircd's version");
  if (IsAnOper(sptr)) {
   SND("-- ");
   SND("Additional IRCop options:");
   SND("OPERCMDS      Show IRCop commands");
  }
  HLP("----");
  return 0;
 }
 else if (what == 2) {
  HDR("-- GXPIRCd help system --");
  SND("User modes:");
  SND("R - only registred nick can message you");
  SND("j - always reject dccallows");
  SND("l - always accept dccallows");
  SND("g - no effect");
  SND("i - invisible (users must know your exact nick to find you)");
  SND("k - see server kill messages");
  SND("s - see server notices");
  SND("w - see WALLOPS messages");
  SND("--");
  if (IsAnOper(sptr)) {
   SND("Additional modes for IRCops:");
   SND("A - server admin");
   SND("a - services admin");
   SND("b - see CHATOPS messages");
   SND("c - see client connections/disconnections");
   SND("e - see DCCALLOW traffic");
   SND("f - see server flood messages");
   SND("g - see GLOBOPS messages");
   SND("h - helper");
   SND("m - see spambot warnings");
   SND("n - see routing notices");
   SND("o - global oper");
   SND("O - local oper");
   SND("y - see STATS and LINKS requests");
   SND("--");
  }
  SND("Usage: /mode [yournick] [[+|-]modes] ");
  HLP("----");
 }
 else if (what == 3) {
  HDR("-- GXPIRCd help system --");
  SND("Channel modes:");
  SND("R - need to have a registred nickname to enter channel");
  SND("M - need to have a registred nickname to talk on channel");
  SND("c - mIRC-style color codes not allowed");
  SND("i - clients must be invited to join");
  SND("m - moderated (only ops and voices may speak)");
  SND("n - no external messages (clients must be in channel to speak)");
  SND("O - only operators may join");
  SND("p - private (does not appear in LIST)");
  SND("R - only registered nicks may join");
  SND("s - secret (does not appear in LIST or WHOIS)");
  SND("t - only ops may set topic");
  SND("b <mask> - (un)ban the specified nick!user@host mask");
  SND("k <key> - key (not) required to join");
  SND("l <limit> - limited to specified number of clients (no parameter for -l)");
  SND("o <nick> - give (take away) chanop status to (from) the specified nick");
  SND("v <nick> - give (take away) voice status to (from) the specified nick");
  SND("--");
  if (IsAnOper(sptr)) {
   SND("Additional Channel Modes for IRCops");
   SND("H - Official Help Channel");
   SND("O - Only opers can join");
   SND("--");
  }
  SND("Usage: /mode [#channel] [[+|-]mode] [parameters]");
  HLP("----");
  return 0;
 }
 else if (what == 4) {
  HDR("-- GXPIRCd help system --");
  SND("WHO:");
  SND("-");
  SND("Usage: WHO [+|-][acghimnsuCM] <arguments>");
  SND("-");
  SND("Flags are... :");
  SND(" - Specified like channel modes,");
  SND(" - Set to a positive check by +, a negative check by -");
  SND("-");
  SND("List of available flags:");
  SND("a          : user is away");
  SND("c <channel>: user is on <channel>,");
  SND("             no wildcards accepted");
  SND("g <gcos/realname>: user has string <gcos> in his GCOS,");
  SND("                   wildcards accepted, oper only");
  SND("h <host>   : user has string <host> in his hostname,");
  SND("             wildcards accepted");
  SND("i <ip>     : user is from <ip>");
  SND("             wildcards accepted,");
  SND("m <usermodes> : user has <usermodes> set on him,");
  SND("                only o/A/a for nonopers");
  SND("n <nick>   : user has string <nick> in his nickname,");
  SND("             wildcards accepted");
  SND("s <server> : user is on server <server>,");
  SND("             wildcards not accepted");
  SND("u <user>   : user has string <user> in his username,");
  SND("             wildcards accepted");
  SND("Behavior flags:");
  SND("C: show first visible channel user is in");
  SND("M: check for user in channels I am a member of");
  HLP("----");
  return 0;
 }
 else if (what == 5) {
  HDR("-- GXPIRCd help system --");
  SND("DCCALLOW:");
  SND("-");
  SND("Usage: DCCALLOW [<+|->nick[,<+|->nick, ...]] [list] [help]");
  SND("-");
  SND("By this command, you can permit someone to send you a file that, ");
  SND("otherwise, would be blocked by the server.");
  SND("-");
  SND("Exemples:");
  SND("To allow user John to send you a .exe file (or any other file):");
  SND("/dccallow +John");
  SND("-");
  SND("Now, John can send you DCC files.");
  SND("Note that if you received an error message, John will have to resend");
  SND("the file to you.");
  SND("-");
  SND("If you want to remove John of your DCCALLOW list, just do:");
  SND("/dccallow -John");
  SND("-");
  SND("To see who is in your DCCALLOW list, just do:");
  SND("/dccallow list");
  HLP("----");
  return 0;
 }
 else if (what == 6) {
  HDR("-- GXPIRCd Help System --");
  SND("Statistical System:");
  SND("-");
  SND("Usage: /stats [flag] <dest>");
  SND("Available flags:");
  SND("I|i - display configured I:lines");
  SND(" L|l [<dest>] - return statistics on linked servers");
  SND(" L|l <nick> - return statistics on specified nick (from uplink server)");
  SND(" M|m - return command usage statistics");
  SND(" N|n - return user connection statistics");
  SND(" O|o - display configured O:lines");
  SND(" P|p - return currently online opers");
  SND(" u [<dest>] - return server uptime information");
  SND(" V|v - return linked servers");
  SND(" Y|y - display configured Y:lines");
  SND(" ? [<dest>] - return server statistics");
  if (IsAnOper(sptr)) {
   SND("--");
   SND("Additional available flags for IRCop");
   SND("c|C - display configured C:lines and N:lines");
   SND("D - display configured Z:lines");
   SND("d - display services-added Z lines");
   SND("e|E - display configured E:lines");
   SND("f|F - display configured F:lines");
   SND("h|H - display configured H:lines");
   SND("k - display temporary K:lines");
   SND("K - display configured K:lines");
   SND("s|S - dump server cache");
   SND("t|T - display traffic statistics");
   SND("U - display configured U:lines");
   SND("z|Z - display memory usage statistics");
  }
  HLP("----");
 }
 else if (what == 7) {
  sendto_one(sptr, ":%s NOTICE %s :-- List of all commands", me.name, sptr->name);
  for (i = 0; msgtab[i].cmd; i++)
   sendto_one(sptr, ":%s NOTICE %s :%s", me.name, sptr->name, msgtab[i].cmd);
  sendto_one(sptr, ":%s NOTICE %s :-- Note: Some commands might be unusable, as they are only "
   "usable by servers", me.name, sptr->name);
  return 0;
 }
 else if (what == 8) {
  HDR("-- GXPIRCd Help System --");
  SND("Oper commands: ");
  SND("-");
  SND("Here's a list of available ircop commands:");
  SND("CHATOPS     PRIVMSG");
  SND("CLOSE       REHASH");
  SND("CONNECT     RESTART");
  SND("DIE         SAMODE");
  SND("GLOBOPS     SET");
  SND("HTM         SQUIT");
  SND("KILL        UNKLINE");
  SND("KLINE       WALLOPS");
  SND("LOCOPS      ZLINE");
  SND("-");
  SND("Usage: HELP opercmds [command]");
  HLP("----");
 }
 else if (what == 811) {
   HDR("-- GXPIRCd Help System --");
   SND("CHATOPS:");
   SND("-");
   SND("Usage: CHATOPS [message]");
   SND("Sends the specified message to +b users");
   HLP("----");
 }
 else if (what == 812) {
   HDR("-- GXPIRCd Help System --");
   SND("CLOSE:");
   SND("-");
   SND("Usage: CLOSE");
   SND("Closes all unknown connections (connections that have not yet registered");
   SND("or are in a state of link creation or tear-down).");
   HLP("----");
 }
 else if (what == 813) {
   HDR("-- GXPIRCd Help System --");
   SND("CONNECT:");
   SND("-");
   SND("Usage: CONNECT [server] <port> <dest>");
   SND("Order connection between two servers");
   SND("If port and dest aren't specified, the connection is local.");
   SND("Otherwise, it orders another server to connect to a server.");
   HLP("----");
 }
 else if (what == 814) {
   HDR("-- GXPIRCd Help System --");
   SND("DIE:");
   SND("Usage: DIE <password>");
   SND("Close the IRCd. Password is mandatory if there is a X line");
   SND("in your server's configuration.");
   HLP("----");
 }
 else if (what == 815) {
   HDR("-- GXPIRCd Help System --");
   SND("GLOBOPS:");
   SND("Usage: GLOBOPS :<message>");
   SND("Send a message to all +g opers.");
   HLP("----");
 }
 else if (what == 816) {
   HDR("-- GXPIRCd Help System --");
   SND("HTM:");
   SND("Usage: HTM <option> <parameter>");
   SND("Sets or displays high traffic mode settings");
   SND("-");
   SND("Available options:");
   SND("ON       - force HTM on");
   SND("OFF      - force HTM off");
   SND("QUIET    - disable announcements when HTM");
   SND("           is entered automatically");
   SND("NOISY    - enable announcements when HTM");
   SND("           is entered automatically");
   SND("TO <max> - sets the HTM threshold to <max> Kbyte/s");
   HLP("----");
 }
 else if (what == 817) {
   HDR("-- GXPIRCd Help System --");
   SND("KILL:");
   SND("Usage: KILL nick1, nick2, ..., nick18 :<reason>");
   SND("Disconnects from the network.");
   SND("-");
   HLP("----");
 }
 else if (what == 818) {
   HDR("-- GXPIRCd Help System --");
   SND("KLINE:");
   SND("Usage: KLINE <minutes> [mask] :<reason>");
   SND("Add a local ban of the user@host mask. If minutes");
   SND("is not supplied, default kline time is used.");
   HLP("----");
 }
 else if (what == 819) {
   HDR("-- GXPIRCd Help System --");
   SND("LOCOPS:");
   SND("Usage: LOCOPS :[message]");
   SND("Sends a message to all operators on this server");
   HLP("----");
 }
 else if (what == 820) {
   HDR("-- GXPIRCd Help System --");
   SND("PRIVMSG:");
   SND("Usage: PRIVMSG [$[servermask]] [#[hostmask]] :[message]");
   SND("If $ is used: Sends a message to all users on all servers");
   SND("              matching servermask");
   SND("If # is used: Sends a message to all users that match");
   SND("              hostmask");
   HLP("----");
 }
 else if (what == 821) {
   HDR("-- GXPIRCd Help System --");
   SND("REHASH:");
   SND("Usage: REHASH <option>");
   SND("If rehash is used with no option, it simply rehash");
   SND("the server's configuration file.");
   SND("-");
   SND("Available options:");
   SND("DNS     - flush DNS cache and reread /etc/resolv.conf");
   SND("SZLINES - clear all services-set Z lines");
   SND("TKLINES - clear all temporary K lines");
   SND("GC      - garbage collect the memory block heaps");
   SND("MOTD    - reload the message of the day");
   SND("IP      - rehash the IP hashtable");
   SND("AKILLS  - rehash the autokill hashtable");
   HLP("----");
 }
 else if (what == 822) {
   HDR("-- GXPIRCd Help System --");
   SND("RESTART:");
   SND("Usage: RESTART <password>");
   SND("Restart the server. Password should be supplied if there's");
   SND("a X line in the server's configuration file.");
   HLP("----");
 }
 else if (what == 823) {
   HDR("-- GXPIRCd Help System --");
   SND("SAMODE:");
   SND("Usage: SAMODE [#channel] [modes]");
   SND("Force a mode change on a channel. Can be only used by");
   SND("services administrators. Same syntax as the mode command.");
   HLP("----");
 }
 else if (what == 824) {
   HDR("-- GXPIRCd Help System --");
   SND("SET:");
   SND("Usage: SET [option] [parameter]");
   SND("Enable a server admin to change runtime options.");
   SND("List of available options: ");
   SND("MAX       Max clients on the server");
#ifdef FLUD
   SND("FLUDNUM     Max number of CTCP");
   SND("FLUDTIME    Max number of CTCP in time");
   SND("FLUDBLOCK   Time that CTCP are blocked if flood is detected");
#endif
#ifdef ANTI_SPAMBOT
   SND("SPAMNUM     Number of join/part cycle to identify a spambot");
   SND("SPAMTIME    Join/part cycle detected in time");
#endif
#ifdef NO_CHANOPS_WHEN_SPLIT
   SND("SPLITDELAY  Time before a split recovery");
#endif
   HLP("----");
 }
 else if (what == 825) {
   HDR("-- GXPIRCd Help System --");
   SND("SQUIT:");
   SND("Usage: SQUIT [mask]");
   SND("Disconnect the first server matching mask from the network.");
   HLP("----");
 }
 else if (what == 826) {
   HDR("-- GXPIRCd Help System --");
   SND("UNKLINE:");
   SND("Usage: UNKLINE [mask]");
   SND("Removes the specified user@host mask from the server banlist.");
   HLP("----");
 }
 else if (what == 827) {
   HDR("-- GXPIRCd Help System --");
   SND("WALLOPS:");
   SND("Usage: WALLOPS :[message]");
   SND("Sends a message to all +w users.");
   SND("Note: That include non-ircop users.");
   HLP("----");
 }
 else if (what == 828) {
   HDR("-- GXPIRCd Help System -- --");
   SND("ZLINE:");
   SND("Usage: ZLINE [ip] :<reason>");
   SND("Adds a ban for the specified IP to the server.");
   SND("The IP may be in the format xxx.xxx.xxx.xxx");
   SND("or xxx.xxx.xxx.*. <IP> may also be a nickname");
   SND("(the IP will be added by the server).");
   HLP("----");
 }
 else if (what == 994) {
   HDR("-- GXPIRCd Help System -- --");
   SND("Need to know:");
   SND("-");
   SND("This ircd is based on bahamut. It adds many functions to it.");
   SND("Here are some of them:");
   SND("- User mode j/l (Automatically reject/allow dccallows");
   SND("- New help system");
   SND("- Channel mode +H (Official Help Channel)");
   SND("- HELP HELPCHANS - Show official help channels");
   SND("- The limit protocol (show global max users)");
   SND("There are few more, but we can't list all of them !");
   HLP("----");
 }
 else if (what == 995) {
  HDR("-- GXPIRCd Help System -- --");
  SND("Changelog");
  SND("-");
  SND("-= Changes in version 0.5.19: =-");
  SND("-");
  SND("Added HELP WHATSNEW and HELP NEWSTUFF");
  SND("Completly reworked SET (never seen so bad code before)");
  SND("Fixed stats p forever;)");
  SND("Changed +H messages");
  SND("Changed some messages");
  SND("Oper hostmasking whois: is actually bla.bla.bla [255.255.255.0], removed the ip (who cares)");
  SND("-");
  SND("-= Changes in version 0.5.18: =-");
  SND("-");
  SND("Fixed some server messages");
  SND("Changed staff address to staff.globalxp.net");
  SND("Fixed stats p to be real more cool :p");
  SND("-");
  SND("-= Changes in version 0.5.16 and 0.5.17: =-");
  SND("-");
  SND("Changed slightly the version reply");
  SND("Changed some notices");
  SND("Little bug in a notice (on server connect), fixed");
  SND("New stats p (slightly modified from prometheus)");
  SND("HELP VFLAGS, help on version flags");
  SND("Channel mode +H (Official Help Channel) only for opers");
  SND("Moved list ? to the help system");
  SND("HELP helpchans, give a list of the official help channels");
  SND("Minor bug fixes");
  SND("Motd Last Change bug fix (now it's true)");
  SND("Updated c/umodes in s_err.c (RPL_PROTOCOL)");
  SND("Added charset=rfc1459 in RPL_PROTOCOL");
  HLP("----");
 }
 else if (what == 996) {
  extern char serveropts[];

  HDR("-- GXPIRCd Help System -- --");
  SND("Version flags:");
  SND("--");
  SND("What are version flags ?");
  SND("When you do /version, you will most likely see that:");
  SND("GXPIRCd(subway)-0.5(17)-release. agoodserv.globalxp.net :LCiI TS3ow");
  SND("The version flags are what is after the last :");
  SND("--");
  SND("List of version flags:");
  SND("L   Limit Protocol Enabled");
  SND("rl  Limit Protocol Enabled, relay only");
  SND("C   Can use boot flags (command line control)");
  SND("D   Debug mode enabled");
  SND("H   Hub");
  SND("i   Show invisible users in lusers");
  SND("I   Auto +i when a client connects");
  SND("u   IRCII Kludge");
  SND("y   Syslog enabled");
  SND("TS3 Time Stamp Server Protocol v3");
  sendto_one(sptr, ":%s 291 %s :This server's flags are: %s",
   me.name, sptr->name, serveropts);
  HLP("----");
 }
 else if (what == 997) {
  HDR("-- GXPIRCd Help System -- --");
  SND("LIST:");
  SND("Usage: /raw LIST options (on mirc) or /quote LIST options (ircII)");
  SND("--");
  SND("If you don't include any options, the default is to send you the");
  SND("entire unfiltered list of channels. Below are the options you can");
  SND("use, and what channels LIST will return when you use them.");
  SND("--");
  SND(">number  List channels with more than <number> people");
  SND("<number  List channels with less than <number> people");
  SND("C>number List channels created between now and <number> minutes ago");
  SND("C<number List channels created earlier than <number> minutes ago");
  SND("T>number List channels whose topics are older than <number> minutes");
  SND("         (i.e.; they have not changed in the last <number> minutes.");
  SND("T<number List channels whose topics are not older than <number> ");
  SND("minutes.");
  SND("*mask*   List channels that match *mask*");
  SND("!*mask*  List channels that do not match *mask*");
  HLP("----");
  return 0;
 }
 else if (what == 999) {
  HDR("-- GXPIRCd help system --");
  SND("Unknown command");
  SND("For more information, do /raw HELP");
  HLP("----");
  return 0;
 }
 else {
  sendto_one(sptr, ":%s NOTICE %s :-- You are using a HELP command that is still in devlopment",
   me.name, sptr->name);
  return 0;
 }
 return 0;
}

/* m_help
 * Main function of the help system
 * Parameters:
 *  - parv[0]: sender prefix
 *  - parv[1]: option
 *     ...
 *  - parv[n]: option
 */
int m_help(aClient *cptr, aClient *sptr, int parc, char *parv[]) {

 static time_t last_used = 0L;

 if (!MyConnect(sptr)) return 0;

 if(!IsAnOper(sptr)) {
	if ((last_used + HELP_WAIT) > NOW)
	    return 0;
	else
	    last_used = NOW;

 }

 if (parc <= 1) {
  send_help(sptr, 1);
  return 0;
 }

 if (!strcasecmp(parv[1], "umodes")) {
  send_help(sptr, 2);
  return 0;
 }
 else if (!strcasecmp(parv[1], "cmodes")) {
  send_help(sptr, 3);
  return 0;
 }
 else if (!strcasecmp(parv[1], "who")) {
  send_help(sptr, 4);
  return 0;
 }
 else if (!strcasecmp(parv[1], "dccallow")) {
  send_help(sptr, 5);
  return 0;
 }
 else if (!strcasecmp(parv[1], "stats")) {
  send_help(sptr, 6);
  return 0;
 }
 else if (!strcasecmp(parv[1], "cmds")) {
  send_help(sptr, 7);
  return 0;
 }
 else if (!strcasecmp(parv[1], "opercmds") && IsAnOper(sptr)) {
  if (parc == 2) {
   send_help(sptr, 8);
   return 0;
  }
  else if (!strcasecmp(parv[2], "CHATOPS")) {
   send_help(sptr, 811);
   return 0;
  }
  else if (!strcasecmp(parv[2], "CLOSE")) {
   send_help(sptr, 812);
   return 0;
  }
  else if (!strcasecmp(parv[2], "CONNECT")) {
   send_help(sptr, 813);
   return 0;
  }
  else if (!strcasecmp(parv[2], "DIE")) {
   send_help(sptr, 814);
   return 0;
  }
  else if (!strcasecmp(parv[2], "GLOBOPS")) {
   send_help(sptr, 815);
   return 0;
  }
  else if (!strcasecmp(parv[2], "HTM")) {
   send_help(sptr, 816);
   return 0;
  }
  else if (!strcasecmp(parv[2], "KILL")) {
   send_help(sptr, 817);
   return 0;
  }
  else if (!strcasecmp(parv[2], "KLINE")) {
   send_help(sptr, 818);
   return 0;
  }
  else if (!strcasecmp(parv[2], "LOCOPS")) {
   send_help(sptr, 819);
   return 0;
  }
  else if (!strcasecmp(parv[2], "PRIVMSG")) {
   send_help(sptr, 820);
   return 0;
  }
  else if (!strcasecmp(parv[2], "REHASH")) {
   send_help(sptr, 821);
   return 0;
  }
  else if (!strcasecmp(parv[2], "RESTART")) {
   send_help(sptr, 822);
   return 0;
  }
  else if (!strcasecmp(parv[2], "SAMODE")) {
   send_help(sptr, 823);
   return 0;
  }
  else if (!strcasecmp(parv[2], "SET")) {
   send_help(sptr, 824);
   return 0;
  }
  else if (!strcasecmp(parv[2], "SQUIT")) {
   send_help(sptr, 825);
   return 0;
  }
  else if (!strcasecmp(parv[2], "UNKLINE")) {
   send_help(sptr, 826);
   return 0;
  }
  else if (!strcasecmp(parv[2], "WALLOPS")) {
   send_help(sptr, 827);
   return 0;
  }
  else if (!strcasecmp(parv[2], "ZLINE")) {
   send_help(sptr, 828);
   return 0;
  }
  return 0;
 }
 else if (!strcasecmp(parv[1], "newstuff")) {
   send_help(sptr, 994);
   return 0;
 }
 else if (!strcasecmp(parv[1], "whatsnew")) {
   send_help(sptr, 995);
   return 0;
 }
 else if (!strcasecmp(parv[1], "vflags")) {
   send_help(sptr, 996);
   return 0;
 }
 else if (!strcasecmp(parv[1], "list")) {
   send_help(sptr, 997);
   return 0;
 }
 else if (!strcasecmp(parv[1], "helpchans")) {
  send_offhlp(sptr);
  return 0;
 }

 else {
  send_help(sptr, 999);
 }
 return 0;
}

